import os
import sys
import gzip
import stat


def here(*ps):
    """ Return script execution path os.path.join'ed with the given arguments.
    """
    return os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), *ps)


def _list_dir(path, dirs, files, ext):
    for fname in os.listdir(path):
        fpath = os.path.join(path, fname)
        mode = os.stat(fpath).st_mode

        isdir = stat.S_ISDIR(mode)
        isfile = stat.S_ISREG(mode)

        d = dirs and isdir
        f = files and isfile and (ext is None or os.path.splitext(fname)[1] == ext)

        if d or f:
            yield fpath


def list_dir(path='.', dirs=False, files=True, ext=None, recursive=False):
    """ List the contents of a directory.
    If recursive is set to True, list_dir will recurse through sub-dirs as
    well.

    Returns directories if dirs is set to True.
    Returns files if files is set True.
    Returns only files with the given extension if ext is set. Note that
        the extension includes the dot, i.e. '.jpg'.
    """
    if not recursive:
        yield from _list_dir(path, dirs, files, ext)
        return

    yield from _list_dir(path, dirs=False, files=True, ext=ext)

    for d in _list_dir(path, dirs=True, files=False, ext=None):
        if dirs:
            yield d
        yield from list_dir(d, dirs, files, ext, recursive=True)


_open = open


def open(path, mode='r'):
    """ Open a file and return a stream.

    If the file has .gz extension, `gzip.open` is used in place of `open`.
    """
    _, ext = os.path.splitext(path)
    if ext == '.gz':
        return gzip.open(path, mode)

    return _open(path, mode)


def open_file(path, mode='r'):
    return open(path, mode)



from .version import version_info, __version__

__all__ = ['io', 'jsonutil', 'dictutil']

from .io import open, open_file, list_dir, here
from .jsonutil import json_load, json_dump
from .dictutil import get_deep
from .progress import ProgressTracker



import micvbang as mvb


class ProgressTracker(object):
    def __init__(self, it, get_id, fpath=None, flush_freq=0, print_skips_freq=0):
        self.skips = 0
        self._get_id = get_id
        self._it = it

        self._ids = set()

        self._flush_freq = flush_freq
        self._print_skips_freq = print_skips_freq

        self._fpath = fpath
        if fpath is None:
            self._fpath = 'progress.gz'

        try:
            with mvb.open(self._fpath, 'rt') as f:
                self._ids = set(f.read().split('\n'))
        except FileNotFoundError:
            pass

    def iter(self):
        with mvb.open(self._fpath, 'at') as f:
            for i, data in enumerate(self._it):
                id = self._get_id(data)
                if id in self._ids:
                    self.skips += 1
                    if self._print_skips_freq and self.skips % self._print_skips_freq == 0:
                        print(" ... skipped {} ...".format(self.skips))
                    continue

                yield data
                self._ids.add(id)
                f.write("{}\n".format(id))

                if self._flush_freq and i % self._flush_freq == 0:
                    f.flush()



import os
import json

import micvbang as mvb


def json_load(path, none_on_error=False):
    mode = 'r'

    _, ext = os.path.splitext(path)
    if ext == '.gz':
        mode += 't'

    with mvb.open(path, mode) as f:
        try:
            return json.load(f)
        except json.JSONDecodeError as e:
            if none_on_error:
                return None
            raise e


def json_dump(obj, path):
    mode = 'w'

    _, ext = os.path.splitext(path)
    if ext == '.gz':
        mode += 't'

    with mvb.open(path, mode) as f:
        return json.dump(obj, f)



import setuptools

import micvbang

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="micvbang",
    version=micvbang.__version__,
    author="Michael Bang",
    author_email="mic@vbang.dk",
    description="micvbang's often used utility functions",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/micvbang/micvbang-python",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
    ),
)



def get_deep(obj, path, separator='.'):
    """ Retrieve the value denoted by path in obj.

    This is done by recursively calling obj.get([path_head], None) until the path is traversed,
    until a value on the path does not have a callable `get` method, or until the next step in
    the path does not exist. In case the full path cannot be traversed, None is returned.
    """
    if not callable(getattr(obj, 'get', None)):
        return None

    head, *tail = path.split(separator)

    if len(tail) == 0:
        return obj.get(head, None)

    return get_deep(obj.get(head, None), separator.join(tail), separator)
